"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.finalizeBehavior = exports.getBehaviorArgs = exports.behaviorGraph = exports.noRegisterSetBehaviorValAction = exports.setBehaviorValAction = exports.setBehaviorValArgs = exports.setBehaviorVal = exports.getBehaviorVal = exports.setBehaviorMut = exports.createBehavior = void 0;
const util_1 = require("../util");
const Behavior_1 = require("../types/Behavior");
/**
 * Creates a new behavior.
 * Usage: createBehavior!(name: string);
 */
exports.createBehavior = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        if (state.graphgame.objects.hasOwnProperty(name))
            throw new Error("A behavior with the name \"" + name + "\" already exists!");
        state.graphgame.behaviors[name] = new Behavior_1.Behavior(name);
        return "";
    }
};
/**
 * Marks a behavior's variable as mutable (changable after compilation).
 * Usage: setBehaviorMut!(name: string, variableName: string);
 */
exports.setBehaviorMut = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        const varName = (0, util_1.getString)(args, state, 1, "A variable name is required!");
        (0, util_1.behaviorCheck)(state, name);
        state.graphgame.behaviors[name].add((id) => `setMut!(${id}, "${(0, util_1.getFullVariableName)(varName, name)}");`);
        return "";
    }
};
/**
 * Get the value of a behavior's variable.
 * Usage: getBehaviorVal!(name: string, variableName: string)
 */
exports.getBehaviorVal = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.expressionCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        const varName = (0, util_1.getString)(args, state, 1, "A variable name is required!");
        (0, util_1.behaviorCheck)(state, name);
        return `getVal!(${state.graphgame.lastObjectBehaviorId}, "${(0, util_1.getFullVariableName)(varName, name)}")`;
    }
};
/**
 * Set the value of a behavior's variable (during compilation). This must be used before a variable is marked as mutable.
 * Usage: setBehaviorVal!(name: string, variableName: string, val: number);
 */
exports.setBehaviorVal = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        const varName = (0, util_1.getString)(args, state, 1, "A variable name is required!");
        const val = (0, util_1.getNum)(args, state, 2, "A value is required!");
        (0, util_1.behaviorCheck)(state, name);
        state.graphgame.behaviors[name].add((id) => `setVal!(${id}, "${(0, util_1.getFullVariableName)(varName, name)}", ${val});`);
        return "";
    }
};
/**
 * Set the value of a behavior's variable (during compilation) using an argument that was passed to the behavior when it was added to the GameObject. This must be used before a variable is marked as mutable.
 * Usage: setBehaviorValArgs!(name: string, variableName: string, arg: number);
 */
exports.setBehaviorValArgs = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        const varName = (0, util_1.getString)(args, state, 1, "A variable name is required!");
        const idx = (0, util_1.getNum)(args, state, 2, "An argument index is required!");
        const defaultVal = (0, util_1.getNum)(args, state, 3);
        (0, util_1.behaviorCheck)(state, name);
        state.graphgame.behaviors[name].add((id) => {
            let val;
            if (state.graphgame.lastObjectBehaviorArgs.length <= idx) {
                if (defaultVal == null)
                    throw new Error("This behavior requires at least " + (idx + 1) + " arguments!");
                else
                    val = defaultVal;
            }
            else {
                val = state.graphgame.lastObjectBehaviorArgs[idx];
            }
            if (typeof (val) === "string")
                val = `"${val}"`;
            else
                val = val.toString();
            return `setVal!(${id}, "${(0, util_1.getFullVariableName)(varName, name)}", ${val});`;
        });
        return "";
    }
};
/**
 * Set the value of a behavior's variable on update (during runtime). This must be used after a variable is marked as mutable.
 * Usage: setBehaviorValAction!(name: string, variableName: string, body: ActionBody);
 */
exports.setBehaviorValAction = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        const varName = (0, util_1.getString)(args, state, 1, "A variable name is required!");
        const body = (0, util_1.getString)(args, state, 2, "An action body is required!");
        (0, util_1.behaviorCheck)(state, name);
        state.graphgame.behaviors[name].addPost((id) => `setValAction!(${id}, "${(0, util_1.getFullVariableName)(varName, name)}", {const ${(0, util_1.getShortVariableName)(varName)} = ${(0, util_1.getFullVariableName)(varName, name)};${body}});`);
        return "";
    }
};
/**
 * Create an action that sets the value of a behavior's variable (during runtime). This must be ran manually. This must be used after a variable is marked as mutable.
 * Usage: noRegisterSetBehaviorValAction!(name: string, variableName: string, body: ActionBody, actionName: string);
 */
exports.noRegisterSetBehaviorValAction = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        const varName = (0, util_1.getString)(args, state, 1, "A variable name is required!");
        const body = (0, util_1.getString)(args, state, 2, "An action body is required!");
        const actionName = (0, util_1.getString)(args, state, 3);
        (0, util_1.behaviorCheck)(state, name);
        state.graphgame.behaviors[name].addPost((id) => `noRegisterSetValAction!(${id}, "${(0, util_1.getFullVariableName)(varName, name)}", {const ${(0, util_1.getShortVariableName)(varName)} = ${(0, util_1.getFullVariableName)(varName, name)};${body}}${actionName ? ", \"" + actionName + "\"" : ""});`);
        return "";
    }
};
/**
 * Create a graph of this object.
 * If only one body is defined, the graph will use 1=body1, and body1=body2 otherwise.
 * Usage: behaviorGraph!(name: string, body1: ActionBody, operator?: string, body2?: ActionBody);
 */
exports.behaviorGraph = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        const body1 = (0, util_1.getString)(args, state, 1, "An action body is required!");
        const operator = (0, util_1.getString)(args, state, 2);
        const body2 = (0, util_1.getString)(args, state, 3);
        (0, util_1.behaviorCheck)(state, name);
        state.graphgame.behaviors[name].addPost((id, idx) => {
            return `inline function g_raphgamepost${name}a${id}a${idx}a1(x, y) {
                ${body2 ? body1 : "state = 1;"}
            }
            inline function g_raphgamepost${name}a${id}a${idx}a2(x, y) {
                ${body2 ? body2 : body1}
            }
            graph { g_raphgamepost${name}a${id}a${idx}a1(x, y) } ${operator || "="} { g_raphgamepost${name}a${id}a${idx}a2(x, y) };`;
        });
        return "";
    }
};
/**
 * Gets an argument that was passed to the behavior when it was added to the GameObject.
 * Usage getBehaviorArgs!(arg: number);
 */
exports.getBehaviorArgs = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.expressionCheck)(context);
        const idx = (0, util_1.getNum)(args, state, 0, "An argument index is required!");
        if (state.graphgame.lastObjectBehaviorArgs.length <= idx)
            throw new Error("This behavior requires at least " + (idx + 1) + " arguments!");
        let val = state.graphgame.lastObjectBehaviorArgs[idx];
        if (typeof (val) === "string")
            val = `"${val}"`;
        else
            val = val.toString();
        return val;
    }
};
/**
 * Finalize the behavior. This must be the last template called on the behavior.
 * Usage: finalizeBehavior!(name: string);
 */
exports.finalizeBehavior = {
    function: (args, state, context) => {
        (0, util_1.ensureState)(state);
        (0, util_1.outerCheck)(context);
        const name = (0, util_1.getString)(args, state, 0, "A behavior name is required!").trim().toLowerCase();
        (0, util_1.behaviorCheck)(state, name);
        state.graphgame.behaviors[name].finalize();
        return "";
    }
};
//# sourceMappingURL=component.js.map